'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.formProviderReducer = formProviderReducer;

var _provider = require('./provider.actions');

var _validator = require('../validator');

var INTEGER_REGEX = /^-?\d+?$/;
var FLOAT_REGEX = /^-?\d+(\.\d+)?$/;
var EMAIL_REGEX = /\S+@\S+\.\S+/;

function formProviderReducer() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : { forms: {} };
    var action = arguments[1];

    var type = action.type;
    var payload = action.payload;
    var EVENTS = _provider.REACT_FORMCTRL.EVENTS;
    switch (type) {
        case EVENTS.REGISTER_FORM:
            return onRegisterForm(state, payload.form);
        case EVENTS.UNREGISTER_FORM:
            return onUnregisterForm(state, payload.form);
        case EVENTS.REGISTER_FIELD:
            return onRegisterField(state, payload.form, payload.field, payload.fieldCtrl);
        case EVENTS.UNREGISTER_FIELD:
            return onUnregisterField(state, payload.form, payload.field);
        case EVENTS.FIELD_CHANGED:
            return onFieldChanged(state, payload.form, payload.field, payload.value, payload.files);
        case EVENTS.FIELD_PROPS_CHANGED:
            return onFieldPropsChanged(state, payload.form, payload.field, payload.props);
        case EVENTS.FIELD_BLURRED:
            return onFieldBlurred(state, payload.form, payload.field);
        case EVENTS.FORM_SUBMITED:
            return onFormSubmited(state, payload.form, payload.formRef);
        case EVENTS.FORM_RESETED:
            return onFormReseted(state, payload.form);
        case EVENTS.REGISTER_VALIDATORS:
            return _extends({}, state, { validators: (0, _validator.combineValidators)(payload.validators) });
        default:
            return state;
    }
}

function onRegisterForm(state, formName) {
    var forms = _extends({}, state.forms);
    if (forms[formName]) {
        var form = forms[formName];
        form.__instances++;
    } else {
        forms[formName] = {
            __instances: 1,
            formName: formName,
            valid: true,
            invalid: false,
            untouched: true,
            touched: false,
            pristine: true,
            dirty: false,
            unchanged: true,
            changed: false,
            fields: {},
            values: {},
            files: {}
        };
    }
    return { forms: forms };
}

function onRegisterField(state, formName, fieldName, fieldCtrl) {
    if (state.forms[formName]) {
        var forms = _extends({}, state.forms);
        var form = forms[formName];
        if (form.fields[fieldName]) {
            fieldCtrl.__instances = form.fields[fieldName].__instances + 1;
        } else {
            fieldCtrl.__instances = 1;
        }
        updateFieldCtrl(state, formName, fieldCtrl, fieldCtrl.initialValue);
        form.fields[fieldName] = fieldCtrl;
        form.values[fieldName] = fieldCtrl.value;
        updateFormCtrl(formName, form);
        return { forms: forms };
    } else {
        console.warn('No form instance with name "' + formName + '" to register field "' + fieldName + '".');
    }
    return state;
}

function onUnregisterForm(state, formName) {
    var forms = _extends({}, state.forms);
    var form = forms[formName];
    if (form.__instances > 1) {
        form.__instances--;
    } else {
        delete forms[formName];
    }
    return { forms: forms };
}

function onUnregisterField(state, formName, fieldName) {
    if (state.forms[formName]) {
        if (state.forms[formName].fields[fieldName]) {
            var forms = _extends({}, state.forms);
            var form = forms[formName];
            var field = form.fields[fieldName];
            if (field.__instances > 1) {
                field.__instances--;
            } else {
                delete form.fields[fieldName];
            }
            return { forms: forms };
        }
    }
    return state;
}

function onFieldChanged(state, formName, fieldName, value, files) {
    if (state.forms[formName]) {
        if (state.forms[formName].fields[fieldName]) {
            var forms = _extends({}, state.forms);
            var form = forms[formName];
            var fieldCtrl = form.fields[fieldName];
            fieldCtrl.dirty = true;
            fieldCtrl.pristine = false;
            updateFieldCtrl(state, formName, fieldCtrl, value, files);
            form.values[fieldName] = value;
            form.files[fieldName] = files;
            updateFormCtrl(formName, form);
            return { forms: forms };
        }
    }
    return state;
}

function onFieldPropsChanged(state, formName, fieldName, props) {
    var forms = _extends({}, state.forms);
    var form = forms[formName];
    var field = form.fields[fieldName];
    field.props = props;
    updateFieldCtrl(state, formName, field, field.value);
    updateFormCtrl(formName, form);
    return { forms: forms };
}

function onFieldBlurred(state, formName, fieldName) {
    var forms = _extends({}, state.forms);
    var form = forms[formName];
    var field = form.fields[fieldName];
    field.touched = true;
    field.untouched = false;
    updateFormCtrl(formName, form);
    return { forms: forms };
}

function onFormSubmited(state, formName, formRef) {
    return state;
}

function onFormReseted(state, formName) {
    var forms = _extends({}, state.forms);
    var form = forms[formName];
    Object.keys(form.fields).forEach(function (fieldName) {
        var field = form.fields[fieldName];
        updateFieldCtrl(state, formName, field, field.initialValue);
        field.touched = false;
        field.untouched = true;
        field.dirty = false;
        field.pristine = true;
        form.values[fieldName] = field.initialValue;
    });
    updateFormCtrl(formName, form);
    return { forms: forms };
}

function updateFormCtrl(formName, form) {
    form.valid = true;
    form.invalid = false;
    form.untouched = true;
    form.touched = false;
    form.pristine = true;
    form.dirty = false;
    form.unchanged = true;
    form.changed = false;
    Object.keys(form.fields).forEach(function (fieldName) {
        var field = form.fields[fieldName];
        if (!field.valid) form.valid = false;
        if (field.invalid) form.invalid = true;
        if (!field.untouched) form.untouched = false;
        if (field.touched) form.touched = true;
        if (!field.pristine) form.pristine = false;
        if (field.dirty) form.dirty = true;
        if (!field.unchanged) form.unchanged = false;
        if (field.changed) form.changed = true;
    });
}

function updateFieldCtrl(state, formName, fieldCtrl, value, files) {
    var initialValue = fieldCtrl.initialValue,
        props = fieldCtrl.props;


    var errors = (0, _validator.validate)(state.validators, state.forms[formName], props, value, files) || [];

    fieldCtrl.value = value;
    fieldCtrl.errors = errors;
    fieldCtrl.unchanged = value === initialValue;
    fieldCtrl.changed = value !== initialValue;
    fieldCtrl.valid = errors.length === 0;
    fieldCtrl.invalid = errors.length > 0;
}